@@ -13,7 +13,9 @@ Trading project:
 # TODO:
 #   add inner class with non-trading utils
 #   debug neural networks
-#   do quick-trade-tuner better
+#   add ftx error ignore
+#   add stop loss adder
+#   add tradingview realtime signals
 
 import itertools
 import random
@@ -39,7 +41,6 @@ from scipy import signal
 from sklearn.preprocessing import MinMaxScaler
 from tensorflow.keras.layers import Dropout, Dense, LSTM
 from tensorflow.keras.models import Sequential, load_model
-from binance.exceptions import BinanceAPIException
 
 
 class Trader(object):
@@ -48,12 +49,6 @@ class Trader(object):
     ticker:   |     str      |  ticker/symbol of chart
     df:       |   dataframe  |  data of chart
     interval: |     str      |  interval of df.
-    one of:
-    1m    30m    3h    1M
-    2m    45m    4h    3M
-    3m    1h     1d    6M
-    5m    90m    3d
-    15m   2h     1w
 
     """
     profit_calculate_coef: float
@@ -89,9 +84,10 @@ class Trader(object):
     __last_take_profit: float
     model: Sequential
     returns_strategy: List[float]
+    sec_interval: int
 
     def __init__(self,
-                 ticker: str = 'AAPL',
+                 ticker: str = 'BTC/USDT',
                  df: pd.DataFrame = pd.DataFrame(),
                  interval: str = '1d',
                  rounding: int = 50,
@@ -103,40 +99,61 @@ class Trader(object):
         self.interval = interval
         if interval == '1m':
             self.profit_calculate_coef = 1 / (60 * 24 * 365)
+            self.sec_interval = 60
         elif interval == '2m':
             self.profit_calculate_coef = 1 / (30 * 24 * 365)
+            self.sec_interval = 120
         elif interval == '3m':
             self.profit_calculate_coef = 1 / (20 * 24 * 365)
+            self.sec_interval = 180
         elif interval == '5m':
             self.profit_calculate_coef = 1 / (12 * 24 * 365)
+            self.sec_interval = 300
         elif interval == '15m':
             self.profit_calculate_coef = 1 / (4 * 24 * 365)
+            self.sec_interval = 15 * 60
         elif interval == '30m':
             self.profit_calculate_coef = 1 / (2 * 24 * 365)
+            self.sec_interval = 60 * 30
         elif interval == '45m':
             self.profit_calculate_coef = 1 / (32 * 365)
+            self.sec_interval = 60 * 45
         elif interval == '1h':
             self.profit_calculate_coef = 1 / (24 * 365)
+            self.sec_interval = 60 * 60
         elif interval == '90m':
             self.profit_calculate_coef = 1 / (18 * 365)
+            self.sec_interval = 60 * 90
         elif interval == '2h':
             self.profit_calculate_coef = 1 / (12 * 365)
+            self.sec_interval = 60 * 60 * 2
         elif interval == '3h':
             self.profit_calculate_coef = 1 / (8 * 365)
+            self.sec_interval = 60 * 60 * 3
         elif interval == '4h':
             self.profit_calculate_coef = 1 / (6 * 365)
+            self.sec_interval = 60 * 60 * 4
+        elif interval == '12h':
+            self.profit_calculate_coef = 1 / (2 * 365)
+            self.sec_interval = 60 * 60 * 12
         elif interval == '1d':
             self.profit_calculate_coef = 1 / 365
+            self.sec_interval = 60 * 60 * 24
         elif interval == '3d':
             self.profit_calculate_coef = 1 / (365 / 3)
+            self.sec_interval = 86400 * 3
         elif interval == '1w':
             self.profit_calculate_coef = 1 / 52
+            self.sec_interval = 86400 * 7
         elif interval == '1M':
             self.profit_calculate_coef = 1 / 12
+            self.sec_interval = 86400 * 30
         elif interval == '3M':
             self.profit_calculate_coef = 1 / 4
+            self.sec_interval = 86400 * 90
         elif interval == '6M':
             self.profit_calculate_coef = 1 / 2
+            self.sec_interval = 86400 * 180
         else:
             raise ValueError(f'I N C O R R E C T   I N T E R V A L; {interval}')
         self._regression_inputs = utils.REGRESSION_INPUTS
@@ -205,27 +222,6 @@ class Trader(object):
         ema3 = ta.trend.ema_indicator(ema2, periods)
         return pd.Series(3 * ema.values - 3 * ema2.values + ema3.values)
 
-    def get_linear(self, dataset) -> np.ndarray:
-        """
-        linear data. mean + (mean diff * n)
-        """
-        mean_diff: float
-        data: pd.DataFrame = pd.DataFrame(dataset)
-
-        mean: float = float(data.mean())
-        mean_diff = float(data.diff().mean())
-        start: float = mean - (mean_diff * (len(data) / 2))
-        end: float = start + (mean - start) * 2
-
-        length: int = len(data)
-        return_list: List[float] = []
-        mean_diff = (end - start) / length
-        i: int
-        for i in range(length):
-            return_list.append(start + mean_diff * i)
-        utils.logger.debug(f'in linear: self.mean_diff={mean_diff}')
-        return np.array(return_list)
-
     def __get_stop_take(self, sig: utils.PREDICT_TYPE) -> Dict[str, float]:
         """
         calculating stop loss and take profit.
@@ -921,7 +917,6 @@ class Trader(object):
                 if bet > deposit:
                     bet = deposit
                 open_price = price
-                bet *= credit_lev
                 deposit -= bet * (commission / 100)
                 if bet > deposit:
                     bet = deposit
@@ -934,8 +929,6 @@ class Trader(object):
                 no_order = False
                 exit_take_stop = False
 
-            if not e:
-                diff = 0.0
             if min(stop_loss, take_profit) < price < max(stop_loss, take_profit):
                 diff = data_column[e] - data_column[e - 1]
             else:
@@ -960,12 +953,12 @@ class Trader(object):
             elif sig == utils.EXIT:
                 diff = 0.0
             if not no_order:
-                deposit += bet * diff / open_price
+                deposit += bet * credit_lev * diff / open_price
             no_order = exit_take_stop
             self.deposit_history.append(deposit)
             oldsig = sig
 
-        self.linear = self.get_linear(self.deposit_history)
+        self.linear = utils.get_linear(self.deposit_history)
         lin_calc_df = pd.DataFrame(self.linear)
         mean_diff = float(lin_calc_df.diff().mean())
         self.year_profit = mean_diff / self.profit_calculate_coef
@@ -1079,6 +1072,7 @@ winrate: {self.winrate}%"""
                         opacity=utils.SCATTER_ALPHA))
             if show:
                 self.fig.show()
+
         return self.backtest_out
 
     def set_pyplot(self,
@@ -1230,18 +1224,12 @@ winrate: {self.winrate}%"""
     def get_trading_predict(self,
                             trading_on_client: bool = False,
                             bet_for_trading_on_client: float = np.inf,
-                            second_symbol_of_ticker: str = 'None',
-                            rounding_bet: int = 4,
-                            coin_lotsize_division=True,
-                            *args,
-                            **kwargs
+                            coin_lotsize_division=True
                             ) -> Dict[str, typing.Union[str, float]]:
         """
         predict and trading.
 
         :param coin_lotsize_division: If for your api you specify the size of the bet in a coin, which is not in which you have a deposit, specify this parameter in the value: True. Otherwise: False, in Binance's case this is definitely the first case (True). If errors occur, try specifying the first ticker symbol instead of the second.
-        :param rounding_bet: maximum permissible accuracy with your api. Bigger than 0
-        :param second_symbol_of_ticker: BTCUSDT -> USDT, for calculate bet. As deposit
         :param trading_on_client: trading on real client
         :param bet_for_trading_on_client: standard: all deposit
         :return: dict with prediction
@@ -1273,7 +1261,7 @@ winrate: {self.winrate}%"""
                     self.client.exit_last_order()
 
                 else:
-                    _moneys_ = self.client.get_balance_ticker(second_symbol_of_ticker)
+                    _moneys_ = self.client.get_balance_ticker(self.ticker.split('/')[1])
                     ticker_price = self.client.get_ticker_price(self.ticker)
                     if coin_lotsize_division:
                         _moneys_ /= ticker_price
@@ -1287,10 +1275,7 @@ winrate: {self.winrate}%"""
 
                     self.client.order_create(predict,
                                              self.ticker,
-                                             bet,
-                                             credit_leverage=credit_leverage,
-                                             rounding_bet=rounding_bet,
-                                             _moneys_=_moneys_)
+                                             bet)
                     self.__exit_order__ = False
         return {
             'predict': predict,
@@ -1302,32 +1287,26 @@ winrate: {self.winrate}%"""
 
     def realtime_trading(self,
                          strategy,
-                         ticker: str = 'AAPL',
-                         get_data_kwargs: Dict[str, typing.Any] = {},
-                         sleeping_time: float = 60.0,
+                         ticker: str = 'BTC/USDT',
                          print_out: bool = True,
                          trading_on_client: bool = False,
                          bet_for_trading_on_client: float = np.inf,
-                         second_symbol_of_ticker: str = 'None',
-                         rounding_bet: int = 4,
                          coin_lotsize_division: bool = True,
                          ignore_exceptions: bool = True,
                          print_exc: bool = True,
+                         wait_sl_tp_checking: float = 5,
                          *strategy_args,
                          **strategy_kwargs):
         """
-        :param print_exc: print binance exceptions in while loop
+        :param wait_sl_tp_checking: sleeping time after stop-loaa and take-profit checking (seconds)
+        :param print_exc: print  exceptions in while loop
         :param ignore_exceptions: ignore binance exceptions in while loop
         :param coin_lotsize_division: If for your api you specify the size of the bet in a coin, which is not in which you have a deposit, specify this parameter in the value: True. Otherwise: False, in Binance's case this is definitely the first case (True). If errors occur, try specifying the first ticker symbol instead of the second.
         :param ticker: ticker for trading.
         :param strategy: trading strategy.
-        :param get_data_kwargs: named arguments to self.client.get_data WITHOUT TICKER.
-        :param sleeping_time: sleeping time / timeframe in seconds.
         :param print_out: printing.
         :param trading_on_client: trading on client
         :param bet_for_trading_on_client: trading bet, standard: all deposit
-        :param second_symbol_of_ticker: USDUAH -> UAH
-        :param rounding_bet: maximum accuracy for trading
         :param strategy_kwargs: named arguments to -strategy.
         :param strategy_args: arguments to -strategy.
         """
@@ -1338,14 +1317,12 @@ winrate: {self.winrate}%"""
             __now__ = time.time()
             while True:
                 try:
-                    self.df = self.client.get_data(self.ticker, **get_data_kwargs).reset_index(drop=True)
+                    self.df = self.client.get_data_historical(ticker=self.ticker, limit=1000, interval=self.interval)
                     strategy(*strategy_args, **strategy_kwargs)
 
                     prediction = self.get_trading_predict(
                         trading_on_client=trading_on_client,
                         bet_for_trading_on_client=bet_for_trading_on_client,
-                        second_symbol_of_ticker=second_symbol_of_ticker,
-                        rounding_bet=rounding_bet,
                         coin_lotsize_division=coin_lotsize_division)
 
                     index = f'{self.ticker}, {time.ctime()}'
@@ -1370,18 +1347,19 @@ winrate: {self.winrate}%"""
                                 self.realtime_returns[index] = prediction
                                 if trading_on_client:
                                     self.client.exit_last_order()
-                        if not (time.time() < (__now__ + sleeping_time)):
+                        time.sleep(wait_sl_tp_checking)
+                        if not (time.time() < (__now__ + self.sec_interval)):
                             self._old_predict = utils.convert_signal_str(self.returns[-1])
-                            __now__ += sleeping_time
+                            __now__ += self.sec_interval
                             break
-                except BinanceAPIException as binance_exc:
+                except Exception as exc:
                     if ignore_exceptions:
                         if print_exc:
-                            print(binance_exc)
+                            print(exc)
                         utils.logger.error('error :(', exc_info=True)
                         continue
                     else:
-                        raise binance_exc
+                        raise exc
 
         except Exception as e:
             utils.logger.critical('error :(', exc_info=True)
@@ -1603,71 +1581,72 @@ winrate: {self.winrate}%"""
         low = self.df['Low']
         close = self.df['Close']
 
-        patterns = map(utils.ta_lib_collider_all,
-                       [
-                           talib.CDL2CROWS(open_, high, low, close),
-                           talib.CDL3BLACKCROWS(open_, high, low, close),
-                           talib.CDL3INSIDE(open_, high, low, close),
-                           talib.CDL3LINESTRIKE(open_, high, low, close),
-                           talib.CDL3OUTSIDE(open_, high, low, close),
-                           talib.CDL3STARSINSOUTH(open_, high, low, close),
-                           talib.CDL3WHITESOLDIERS(open_, high, low, close),
-                           talib.CDLABANDONEDBABY(open_, high, low, close),
-                           talib.CDLADVANCEBLOCK(open_, high, low, close),
-                           talib.CDLBELTHOLD(open_, high, low, close),
-                           talib.CDLBREAKAWAY(open_, high, low, close),
-                           talib.CDLCLOSINGMARUBOZU(open_, high, low, close),
-                           talib.CDLCONCEALBABYSWALL(open_, high, low, close),
-                           talib.CDLCOUNTERATTACK(open_, high, low, close),
-                           talib.CDLDARKCLOUDCOVER(open_, high, low, close),
-                           talib.CDLDOJI(open_, high, low, close),
-                           talib.CDLDOJISTAR(open_, high, low, close),
-                           talib.CDLDRAGONFLYDOJI(open_, high, low, close),
-                           talib.CDLENGULFING(open_, high, low, close),
-                           talib.CDLEVENINGDOJISTAR(open_, high, low, close),
-                           talib.CDLEVENINGSTAR(open_, high, low, close),
-                           talib.CDLGAPSIDESIDEWHITE(open_, high, low, close),
-                           talib.CDLGRAVESTONEDOJI(open_, high, low, close),
-                           talib.CDLHAMMER(open_, high, low, close),
-                           talib.CDLHANGINGMAN(open_, high, low, close),
-                           talib.CDLHARAMI(open_, high, low, close),
-                           talib.CDLHARAMICROSS(open_, high, low, close),
-                           talib.CDLHIGHWAVE(open_, high, low, close),
-                           talib.CDLHIKKAKE(open_, high, low, close),
-                           talib.CDLHIKKAKEMOD(open_, high, low, close),
-                           talib.CDLHOMINGPIGEON(open_, high, low, close),
-                           talib.CDLIDENTICAL3CROWS(open_, high, low, close),
-                           talib.CDLINNECK(open_, high, low, close),
-                           talib.CDLINVERTEDHAMMER(open_, high, low, close),
-                           talib.CDLKICKING(open_, high, low, close),
-                           talib.CDLKICKINGBYLENGTH(open_, high, low, close),
-                           talib.CDLLADDERBOTTOM(open_, high, low, close),
-                           talib.CDLLONGLEGGEDDOJI(open_, high, low, close),
-                           talib.CDLLONGLINE(open_, high, low, close),
-                           talib.CDLMARUBOZU(open_, high, low, close),
-                           talib.CDLMATCHINGLOW(open_, high, low, close),
-                           talib.CDLMATHOLD(open_, high, low, close),
-                           talib.CDLMORNINGDOJISTAR(open_, high, low, close),
-                           talib.CDLMORNINGSTAR(open_, high, low, close),
-                           talib.CDLONNECK(open_, high, low, close),
-                           talib.CDLPIERCING(open_, high, low, close),
-                           talib.CDLRICKSHAWMAN(open_, high, low, close),
-                           talib.CDLRISEFALL3METHODS(open_, high, low, close),
-                           talib.CDLSEPARATINGLINES(open_, high, low, close),
-                           talib.CDLSHOOTINGSTAR(open_, high, low, close),
-                           talib.CDLSHORTLINE(open_, high, low, close),
-                           talib.CDLSPINNINGTOP(open_, high, low, close),
-                           talib.CDLSTALLEDPATTERN(open_, high, low, close),
-                           talib.CDLSTICKSANDWICH(open_, high, low, close),
-                           talib.CDLTAKURI(open_, high, low, close),
-                           talib.CDLTASUKIGAP(open_, high, low, close),
-                           talib.CDLTHRUSTING(open_, high, low, close),
-                           talib.CDLTRISTAR(open_, high, low, close),
-                           talib.CDLUNIQUE3RIVER(open_, high, low, close),
-                           talib.CDLUPSIDEGAP2CROWS(open_, high, low, close),
-                           talib.CDLXSIDEGAP3METHODS(open_, high, low, close)
-                       ]
-                       )
+        patterns = map(
+            utils.ta_lib_collider_all,
+            [
+                talib.CDL2CROWS(open_, high, low, close),
+                talib.CDL3BLACKCROWS(open_, high, low, close),
+                talib.CDL3INSIDE(open_, high, low, close),
+                talib.CDL3LINESTRIKE(open_, high, low, close),
+                talib.CDL3OUTSIDE(open_, high, low, close),
+                talib.CDL3STARSINSOUTH(open_, high, low, close),
+                talib.CDL3WHITESOLDIERS(open_, high, low, close),
+                talib.CDLABANDONEDBABY(open_, high, low, close),
+                talib.CDLADVANCEBLOCK(open_, high, low, close),
+                talib.CDLBELTHOLD(open_, high, low, close),
+                talib.CDLBREAKAWAY(open_, high, low, close),
+                talib.CDLCLOSINGMARUBOZU(open_, high, low, close),
+                talib.CDLCONCEALBABYSWALL(open_, high, low, close),
+                talib.CDLCOUNTERATTACK(open_, high, low, close),
+                talib.CDLDARKCLOUDCOVER(open_, high, low, close),
+                talib.CDLDOJI(open_, high, low, close),
+                talib.CDLDOJISTAR(open_, high, low, close),
+                talib.CDLDRAGONFLYDOJI(open_, high, low, close),
+                talib.CDLENGULFING(open_, high, low, close),
+                talib.CDLEVENINGDOJISTAR(open_, high, low, close),
+                talib.CDLEVENINGSTAR(open_, high, low, close),
+                talib.CDLGAPSIDESIDEWHITE(open_, high, low, close),
+                talib.CDLGRAVESTONEDOJI(open_, high, low, close),
+                talib.CDLHAMMER(open_, high, low, close),
+                talib.CDLHANGINGMAN(open_, high, low, close),
+                talib.CDLHARAMI(open_, high, low, close),
+                talib.CDLHARAMICROSS(open_, high, low, close),
+                talib.CDLHIGHWAVE(open_, high, low, close),
+                talib.CDLHIKKAKE(open_, high, low, close),
+                talib.CDLHIKKAKEMOD(open_, high, low, close),
+                talib.CDLHOMINGPIGEON(open_, high, low, close),
+                talib.CDLIDENTICAL3CROWS(open_, high, low, close),
+                talib.CDLINNECK(open_, high, low, close),
+                talib.CDLINVERTEDHAMMER(open_, high, low, close),
+                talib.CDLKICKING(open_, high, low, close),
+                talib.CDLKICKINGBYLENGTH(open_, high, low, close),
+                talib.CDLLADDERBOTTOM(open_, high, low, close),
+                talib.CDLLONGLEGGEDDOJI(open_, high, low, close),
+                talib.CDLLONGLINE(open_, high, low, close),
+                talib.CDLMARUBOZU(open_, high, low, close),
+                talib.CDLMATCHINGLOW(open_, high, low, close),
+                talib.CDLMATHOLD(open_, high, low, close),
+                talib.CDLMORNINGDOJISTAR(open_, high, low, close),
+                talib.CDLMORNINGSTAR(open_, high, low, close),
+                talib.CDLONNECK(open_, high, low, close),
+                talib.CDLPIERCING(open_, high, low, close),
+                talib.CDLRICKSHAWMAN(open_, high, low, close),
+                talib.CDLRISEFALL3METHODS(open_, high, low, close),
+                talib.CDLSEPARATINGLINES(open_, high, low, close),
+                talib.CDLSHOOTINGSTAR(open_, high, low, close),
+                talib.CDLSHORTLINE(open_, high, low, close),
+                talib.CDLSPINNINGTOP(open_, high, low, close),
+                talib.CDLSTALLEDPATTERN(open_, high, low, close),
+                talib.CDLSTICKSANDWICH(open_, high, low, close),
+                talib.CDLTAKURI(open_, high, low, close),
+                talib.CDLTASUKIGAP(open_, high, low, close),
+                talib.CDLTHRUSTING(open_, high, low, close),
+                talib.CDLTRISTAR(open_, high, low, close),
+                talib.CDLUNIQUE3RIVER(open_, high, low, close),
+                talib.CDLUPSIDEGAP2CROWS(open_, high, low, close),
+                talib.CDLXSIDEGAP3METHODS(open_, high, low, close)
+            ]
+        )
 
         patterns = map(utils.anti_set_, patterns)
         self.set_open_stop_and_take()
